# -*- coding: utf-8 -*-
# Project : flacon
# File : tracklist.py
# (c) Alexander Sokolov asokol@mail.ru
#
# This file is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This file is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA


import preferences
from PyQt4 import QtCore 
from PyQt4 import QtGui
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyKDE4.kdecore import *
from PyKDE4.kdeui import *
from PyKDE4.kio import *

from configureDialog_ui import *



class ConfigureDialog(QtGui.QDialog):
    generalPageIndex  = 0
    flacPageIndex     = 1
    oggPageIndex      = 2
    mp3PageIndex      = 3
    programsPageIndex = 4
   
   
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.form = Ui_configureDialog()
        self.form.setupUi(self)

        #***********************************************************
        self.connect(self.form.pages, SIGNAL('currentChanged(int)'), self.setPage)
       
        self.setPagesList()
        self.setFlacWidgets()
        self.setOggWidgets()
        self.setMp3Widgets()
        self.setProgramsWidgets()
        
        #***********************************************************
        self.generalLoad()
        self.flacLoad()
        self.oggLoad()
        self.mp3Load()
        self.programsLoad()

        
   #****************************************************************
   #
   #****************************************************************
    def done(self, res):
        if res:
            self.generalWrite()        
            self.flacWrite()
            self.oggWrite()
            self.mp3Write()
            self.programsWrite()
            
        QDialog.done(self, res)


   #****************************************************************
   #
   #****************************************************************
    def show(self, pageIndex=-1):
        if pageIndex<0:
            pageIndex = self.form.pages.currentIndex()
        
        self.form.pagesList.setCurrentRow(pageIndex)
        QDialog.show(self)
        

   #****************************************************************
   #
   #****************************************************************
    def setPage(self, pageIndex):
        self.form.pagesList.setCurrentRow(pageIndex)
        
        
   #****************************************************************
   #
   #****************************************************************
    def setPagesList(self):
        pl = self.form.pagesList
        pl.item(ConfigureDialog.generalPageIndex).setIcon(KIcon('go-home'))
        pl.item(ConfigureDialog.flacPageIndex).setIcon(KIcon('audio-x-flac'))
        pl.item(ConfigureDialog.oggPageIndex).setIcon(KIcon('audio-x-ogg'))
        pl.item(ConfigureDialog.mp3PageIndex).setIcon(KIcon('audio-x-mp3'))
        pl.item(ConfigureDialog.programsPageIndex).setIcon(KIcon('applications-system'))
        
        width = 0
        for i in xrange(0, pl.count()):
            item = pl.item(i)
            width = max(width, pl.fontMetrics().width(item.text()))
        
        width = width + 42
               
        for i in xrange(0, pl.count()):
            item = pl.item(i)
            h =  pl.iconSize().height() + pl.fontMetrics().height() + 8
            item.setSizeHint(QSize(width, h))
        
        
        pl.setMaximumWidth(width + 2 * pl.frameWidth())
        

        
   #****************************************************************
   #
   #****************************************************************
    def setFlacWidgets(self):
        self.form.flacCompressionSpin.setToolTip(self.form.flacCompressionSlider.toolTip())


   #****************************************************************
   #
   #****************************************************************
    def setOggWidgets(self):
        self.form.oggQualitySpin.setToolTip(self.form.oggQualitySlider.toolTip())
        self.form.oggQualityLabel.setToolTip(self.form.oggQualitySlider.toolTip())
        self.connect(self.form.oggQualitySlider, SIGNAL('valueChanged(int)'), self.oggQualitySliderChanged)
        self.connect(self.form.oggQualitySpin, SIGNAL('valueChanged(double)'), self.oggQualitySpinChanged)

    
   #****************************************************************
   #
   #****************************************************************
    def setMp3Widgets(self):
        presets = self.form.mp3PresetCbx

        presets.addItem(self.tr('AVR medium'),        QVariant('avrMedium'))
        presets.addItem(self.tr('AVR standard'),      QVariant('avrStandard'))
        presets.addItem(self.tr('AVR standard fast'), QVariant('avrStandardFast'))
        presets.addItem(self.tr('AVR extreme'),       QVariant('avrExtreme'))
        presets.addItem(self.tr('AVR extreme fast'),  QVariant('avrExtremeFast'))
        presets.addItem(self.tr('CBR insane'),        QVariant('cbrInsane'))
        presets.addItem(self.tr('CBR kbps'),          QVariant('cbrKbps'))
        presets.addItem(self.tr('ABR kbps'),          QVariant('abrKbps'))
        presets.addItem(self.tr('VBR quality'),       QVariant('vbrQuality'))


        bitrates = self.form.mp3BitrateCbx
        bitrates.addItem(self.tr('32 kbps'),  QVariant(32))
        bitrates.addItem(self.tr('40 kbps'),  QVariant(40))
        bitrates.addItem(self.tr('48 kbps'),  QVariant(48))
        bitrates.addItem(self.tr('56 kbps'),  QVariant(56))
        bitrates.addItem(self.tr('64 kbps'),  QVariant(64))
        bitrates.addItem(self.tr('80 kbps'),  QVariant(80))
        bitrates.addItem(self.tr('96 kbps'),  QVariant(96))
        bitrates.addItem(self.tr('112 kbps'), QVariant(112))
        bitrates.addItem(self.tr('128 kbps'), QVariant(128))
        bitrates.addItem(self.tr('160 kbps'), QVariant(160))
        bitrates.addItem(self.tr('192 kbps'), QVariant(192))
        bitrates.addItem(self.tr('224 kbps'), QVariant(224))
        bitrates.addItem(self.tr('256 kbps'), QVariant(256))
        bitrates.addItem(self.tr('320 kbps'), QVariant(320))


        self.connect(self.form.mp3PresetCbx, SIGNAL('currentIndexChanged(int)'), self.mp3PresetCbxCanged)
        self.mp3PresetCbxCanged(self.form.mp3PresetCbx.currentIndex())
    

   #****************************************************************
   #
   #****************************************************************
    def setProgramsWidgets(self):
        self.connect(self.form.progShntoolBtn, SIGNAL('clicked()'), self.openProgFile)
        self.connect(self.form.progMacBtn,     SIGNAL('clicked()'), self.openProgFile)
        self.connect(self.form.progFlacBtn,    SIGNAL('clicked()'), self.openProgFile)
        self.connect(self.form.progOggencBtn,  SIGNAL('clicked()'), self.openProgFile)
        self.connect(self.form.progLameBtn,    SIGNAL('clicked()'), self.openProgFile)
        self.connect(self.form.progWavpackBtn, SIGNAL('clicked()'), self.openProgFile)
        self.connect(self.form.progScanButton, SIGNAL('clicked()'), self.findPrograms)
        
        self.form.progShntoolBtn.setIcon(KIcon('document-open-folder'))
        self.form.progMacBtn.setIcon(KIcon('document-open-folder'))
        self.form.progFlacBtn.setIcon(KIcon('document-open-folder'))
        self.form.progOggencBtn.setIcon(KIcon('document-open-folder'))
        self.form.progLameBtn.setIcon(KIcon('document-open-folder'))
        self.form.progWavpackBtn.setIcon(KIcon('document-open-folder'))
       
       
   #****************************************************************
   #
   #****************************************************************
    def oggQualitySliderChanged(self, value):    
        val = value / 10.0
        self.form.oggQualitySpin.setValue(val)
               

   #****************************************************************
   #
   #****************************************************************
    def oggQualitySpinChanged(self, value):   
        self.form.oggQualitySlider.setValue(value * 10)
        #       -1  0   1   2   3    4    5    6    7    8    9    10   fake
        kbps = [45, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 500, 500] 
        x1 = int(value)
        y1=kbps[x1+1]; 
        y2=kbps[x1+2]
        bitrate = round((y2-y1+0.0) * (value - x1) + y1)
        self.form.oggQualityLabel.setText(QString("(~%1 kbps)").arg(bitrate))

        self.fillOggBitrateComboBox(self.form.oggMinBitrateCbx)
        self.fillOggBitrateComboBox(self.form.oggNormBitrateCbx)
        self.fillOggBitrateComboBox(self.form.oggMaxBitrateCbx)


   #****************************************************************
   #
   #****************************************************************
    def fillOggBitrateComboBox(self, comboBox):
        comboBox.addItem(self.tr('Default'),  QVariant(''))
        comboBox.addItem(self.tr('64 kbps'),  QVariant('64'))
        comboBox.addItem(self.tr('80 kbps'),  QVariant('80'))
        comboBox.addItem(self.tr('96 kbps'),  QVariant('96'))
        comboBox.addItem(self.tr('128 kbps'), QVariant('128'))
        comboBox.addItem(self.tr('160 kbps'), QVariant('160'))
        comboBox.addItem(self.tr('196 kbps'), QVariant('196'))
        comboBox.addItem(self.tr('256 kbps'), QVariant('256'))
        comboBox.addItem(self.tr('350 kbps'), QVariant('350'))


   #****************************************************************
   #
   #****************************************************************
    def mp3PresetCbxCanged(self, index):
        preset = self.form.mp3PresetCbx.itemData(index)
        
        enable = (preset == 'abrKbps' or preset=='cbrKbps')
        self.form.mp3BitrateBox.setEnabled(enable)
        
        enable = (preset == 'vbrQuality')
        self.form.mp3QualityBox.setEnabled(enable)


   #****************************************************************
   #
   #****************************************************************
    def loadComboBox(self, key, comboBox):
        data = qApp.preferences.value(key)
        comboBox.setCurrentIndex(comboBox.findData(data))


   #****************************************************************
   #
   #****************************************************************
    def writeComboBox(self, key, comboBox):
        data = comboBox.itemData(comboBox.currentIndex())
        qApp.preferences.setValue(key, data)

  
   #****************************************************************
   #
   #****************************************************************
    def loadSpinBox(self, key, spinBox):
        if isinstance(spinBox, QDoubleSpinBox):
            (value, ok) = qApp.preferences.value(key).toDouble()
            if ok:  spinBox.setValue(value)

        if isinstance(spinBox, QSpinBox):
            (value, ok) = qApp.preferences.value(key).toInt()
            if ok:  spinBox.setValue(value)


   #****************************************************************
   #
   #****************************************************************
    def writeSpinBox(self, key, spinBox):
         qApp.preferences.setValue(key, spinBox.value())


   #****************************************************************
   #
   #****************************************************************
    def loadSlider(self, key, slider):
        (value, ok) = qApp.preferences.value(key).toInt()
        if ok:
            slider.setValue(value)
    
    
   #****************************************************************
   #
   #****************************************************************
    def writeSlider(self, key, slider):
        qApp.preferences.setValue(key, slider.value())
    
    
   #****************************************************************
   #
   #****************************************************************
    def loadLineEdit(self, key, edit):
        edit.setText(qApp.preferences.value(key).toString())


   #****************************************************************
   #
   #****************************************************************
    def writeLineEdit(self, key, edit):
        qApp.preferences.setValue(key, QVariant(edit.text()))


   #****************************************************************
   #
   #****************************************************************
    def generalLoad(self):
        self.loadSpinBox('Encoder/ThreadCount', self.form.threadsCountSpin)
        

   #****************************************************************
   #
   #****************************************************************
    def generalWrite(self):
        self.writeSpinBox('Encoder/ThreadCount', self.form.threadsCountSpin)

   #****************************************************************
   #
   #****************************************************************
    def flacLoad(self):
        self.loadSlider('Flac/Compression', self.form.flacCompressionSlider)


   #****************************************************************
   #
   #****************************************************************
    def flacWrite(self):
        self.writeSlider('Flac/Compression', self.form.flacCompressionSlider)


   #****************************************************************
   #
   #****************************************************************
    def oggLoad(self):
        self.form.oggUseQualityCheck.setChecked(qApp.preferences.value('Ogg/UseQuality').toBool())
        self.loadSpinBox('Ogg/Quality', self.form.oggQualitySpin)
        self.loadComboBox('Ogg/MinBitrate',  self.form.oggMinBitrateCbx)
        self.loadComboBox('Ogg/NormBitrate', self.form.oggNormBitrateCbx)
        self.loadComboBox('Ogg/MaxBitrate',  self.form.oggMaxBitrateCbx)


   #****************************************************************
   #
   #****************************************************************
    def oggWrite(self):
        qApp.preferences.setValue('Ogg/UseQuality', self.form.oggUseQualityCheck.isChecked())
        self.writeSpinBox('Ogg/Quality', self.form.oggQualitySpin)
        self.writeComboBox('Ogg/MinBitrate',  self.form.oggMinBitrateCbx)
        self.writeComboBox('Ogg/NormBitrate', self.form.oggNormBitrateCbx)
        self.writeComboBox('Ogg/MaxBitrate',  self.form.oggMaxBitrateCbx)


   #****************************************************************
   #
   #****************************************************************
    def mp3Load(self):
        self.loadComboBox('Mp3/Preset',  self.form.mp3PresetCbx)
        self.loadComboBox('Mp3/Bitrate', self.form.mp3BitrateCbx)
        self.loadSpinBox('Mp3/Quality',  self.form.mp3QualitySpin)

   #****************************************************************
   #
   #****************************************************************
    def mp3Write(self):
        self.writeComboBox('Mp3/Preset',  self.form.mp3PresetCbx)
        self.writeComboBox('Mp3/Bitrate', self.form.mp3BitrateCbx)
        self.writeSpinBox('Mp3/Quality',  self.form.mp3QualitySpin)


   #****************************************************************
   #
   #****************************************************************
    def programsLoad(self):
        self.loadLineEdit('Programs/shntool', self.form.progShntoolEdit)
        self.loadLineEdit('Programs/flac',    self.form.progFlacEdit)
        self.loadLineEdit('Programs/mac',     self.form.progMacEdit)
        self.loadLineEdit('Programs/oggenc',  self.form.progOggencEdit)
        self.loadLineEdit('Programs/lame',    self.form.progLameEdit)
        self.loadLineEdit('Programs/wavpack', self.form.progWavpackEdit)


   #****************************************************************
   #
   #****************************************************************
    def programsWrite(self):
        self.writeLineEdit('Programs/shntool', self.form.progShntoolEdit)
        self.writeLineEdit('Programs/flac',    self.form.progFlacEdit)
        self.writeLineEdit('Programs/mac',     self.form.progMacEdit)
        self.writeLineEdit('Programs/oggenc',  self.form.progOggencEdit)
        self.writeLineEdit('Programs/lame',    self.form.progLameEdit)
        self.writeLineEdit('Programs/wavpack', self.form.progWavpackEdit)
        
        
   #****************************************************************
   #
   #****************************************************************
    def openProgFile(self):
        btn = self.sender()
        if btn == self.form.progShntoolBtn:
            edit = self.form.progShntoolEdit
            progName = "shntool"

        elif btn == self.form.progMacBtn:
            edit = self.form.progMacEdit
            progName = "mac"
            
        elif btn == self.form.progFlacBtn:
            edit = self.form.progFlacEdit
            progName = "flac"
            
        elif btn == self.form.progOggencBtn:
            edit = self.form.progOggencEdit
            progName = "oggenc"
            
        elif btn == self.form.progLameBtn:
            edit = self.form.progLameEdit
            progName = "lame"

        elif btn == self.form.progWavpackBtn:
            edit = self.form.progWavpackEdit
            progName = "wavpack"


        flt = progName +"*|" + self.tr("%1 program (%1*)").arg(progName)  + "\n"  \
              "*.*|"         + self.tr("All files (*.*)")       + "\n"

        progFile = KFileDialog.getOpenFileName(KUrl("/usr/bin"), flt, self, self.tr("Select program file"))
        if (progFile):
            edit.setText(progFile)


   #****************************************************************
   #
   #****************************************************************
    def findPrograms(self):
        qApp.preferences.findPrograms()
        self.programsLoad()
        
        