# -*- coding: utf-8 -*-
# Project : flacon
# File : controls.py
# (c) Alexander Sokolov asokol@mail.ru
#
# This file is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This file is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA


from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import *
from PyQt4.QtGui import *


class YearSpinBox(QtGui.QSpinBox):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, parent=None):
        QtGui.QSpinBox.__init__(self, parent)
 
 
   #****************************************************************
   #
   #****************************************************************
    def textFromValue(self, value):
        if ( value == self.minimum() ) : 
          return QtCore.QString("")
        else:  
          return QtGui.QSpinBox.textFromValue(self, value)

       
   #****************************************************************
   #
   #****************************************************************
    def validate(self, text, pos):
        if (text == ''):
            self.setValue(self.minimum())
            return (QtGui.QValidator.Acceptable, pos)
            
        return QtGui.QSpinBox.validate(self, text, pos)            

        
   #****************************************************************
   #
   #****************************************************************
    def text(self):
        return self.textFromValue(self.value())

        
   #****************************************************************
   #
   #****************************************************************
    def setText(self, text):
        if (text):
            self.setValue(int(text))
        else:
            self.setValue(self.minimum())
        

class ActionPushButton(QtGui.QPushButton):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, parent = None):
        QtGui.QPushButton.__init__(self, parent)
        self._defAction = None


   #****************************************************************
   #
   #****************************************************************
    def setDefaultAction (self, action):
        self._defAction = action
        self.connect(action, QtCore.SIGNAL('changed()'), self.actionCnanged)
        self.connect(self, QtCore.SIGNAL('clicked()'), self._clicked)
        self.actionCnanged()


   #****************************************************************
   #
   #****************************************************************
    def actionCnanged(self):
        self.setText(self._defAction.text())
        self.setIcon(self._defAction.icon())
        self.setToolTip(self._defAction.toolTip())
        self.setEnabled(self._defAction.isEnabled())


   #****************************************************************
   #
   #****************************************************************
    def _clicked(self):
        if (self._defAction):
            self._defAction.trigger()
            
            
class ItemsLabel(QtGui.QLabel):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, parent=None):
        QtGui.QLabel.__init__(self, parent)
        self._items = None
        self._currentIndex = -1


   #****************************************************************
   #
   #****************************************************************
    def items(self):
        return self._items

    
   #****************************************************************
   #
   #****************************************************************
    def setItems(self, items):
        self._items = items
        size = QLabel.minimumSize(self)
        width = 0
        for item in self._items:
            width = max(width, self.fontMetrics().width(item))
        
        size.setWidth(width)
        self.setMinimumSize(size)
        

   #****************************************************************
   #
   #****************************************************************
    def currentIndex(self):
        return self._currentIndex


   #****************************************************************
   #
   #****************************************************************
    def setCurrentIndex(self, index):
        self._currentIndex = index
        self.setText(self._items[index])
        size = self.minimumSize()
        
        #self.adjustSize()
        