#!/usr/bin/python
# -*- coding: utf-8 -*-
# Project : flacon
# File : flacon.py
# (c) Alexander Sokolov asokol@mail.ru
# Version 0.2
#
# This file is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This file is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA


import sys
import os
import string
import glob
import re

from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import *
from PyKDE4.kdecore import *
from PyKDE4.kdeui import *
from PyKDE4.kio import *
from mainWindow_ui import Ui_mainWindow

from tracklist import *
from converter import *
from preferences import *
from configureDialog import *

class StateDelegate(QtGui.QItemDelegate):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, parent):
        QtGui.QItemDelegate.__init__(self, parent)


   #****************************************************************
   #
   #****************************************************************
    def paint(self, painter, option, index):
        #if (option.state and QtGui.QStyle.State_Selected):
        #    painter.fillRect(option.rect, option.palette.highlight());
        #QtGui.QItemDelegate.paint(self, painter, option, index)
        
        
        opt = QtGui.QStyleOptionProgressBar()
        opt.rect = option.rect.adjusted(4, 4, -4, -4)
        opt.minimum = 0
        opt.maximum = 100
        opt.progress = index.model().data(index, TrackList.StatusPercentRole)
        s = QVariant(index.model().data(index, TrackList.StatusTextRole)).toString()  
        if (opt.progress < 0):
            opt.text = s
        else:
            opt.text = QString("%1 %2%").arg(s).arg(opt.progress)
            

        if (s != ''):
            QtGui.QApplication.style().drawControl(QtGui.QStyle.CE_ProgressBarContents, opt, painter, None)
            QtGui.QApplication.style().drawControl(QtGui.QStyle.CE_ProgressBarLabel, opt, painter, None)



class Flacon(KXmlGuiWindow):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self):
        KXmlGuiWindow.__init__(self)
        
        self.form = Ui_mainWindow()
        self.centralWidget = QtGui.QWidget(self)
        self.form.setupUi(self.centralWidget)
        self.setCentralWidget(self.centralWidget)
        
        self.trackList = TrackList(self.form.trackListView)
        self.trackList.setView(self.form.trackListView)
        self.trackList.setIcon(KIconLoader().loadIcon('audio-basic', KIconLoader.Toolbar))
    
        self.converter = Converter(self)
        
        self.delegate = StateDelegate(self.form.trackListView)
        
        self.form.trackListView.setModel(self.trackList)
        self.form.trackListView.header().setStretchLastSection(True)
        self.form.trackListView.hideColumn(TrackList.COLUMN_STATUS)
        self.form.trackListView.setItemDelegateForColumn(TrackList.COLUMN_STATUS, self.delegate)
        self.form.trackListView.setContextMenuPolicy(Qt.ActionsContextMenu)
      
        self.setMenu()

        self.connect(self.trackList, SIGNAL('dataChanged(QModelIndex, QModelIndex)'), self.refreshEdits)
        self.connect(self.trackList, SIGNAL('modelReset()'), self.trackListReset)
        self.connect(self.trackList, SIGNAL('tagsChanged()'), self.tagsChanged)
        self.connect(self.form.trackListView.selectionModel(), SIGNAL('currentChanged(QModelIndex, QModelIndex)'), self.refreshEdits)
        
        self.connect(self.form.cueFileEdit, SIGNAL('textEdited(QString)'), self.setCueFile)
        self.connect(self.form.audioFileEdit, SIGNAL('textEdited(QString)'), self.setAudioFile)

        self.connect(self.form.outPatternEdit, SIGNAL('textEdited(QString)'), self.trackList.setPattern)
        self.connect(self.form.outDirEdit,     SIGNAL('textEdited(QString)'), self.trackList.setOutDir)
        self.connect(self.form.outFormatCombo, SIGNAL('currentIndexChanged(int)'), self.setOutFormat)        

        self.connect(self.form.tagGenreEdit,    SIGNAL('textEdited(QString)'),   self.setGenre)
        self.connect(self.form.tagYearEdit,     SIGNAL('valueChanged(QString)'), self.setYear)
        self.connect(self.form.tagArtistEdit,   SIGNAL('textEdited(QString)'),   self.setArtist)
        self.connect(self.form.tagAlbumEdit,    SIGNAL('textEdited(QString)'),   self.setAlbum)
        self.connect(self.form.tagStartNumEdit, SIGNAL('valueChanged(int)'), self.trackList.setStartTrackNum)
        self.connect(self.form.tagDiscIdEdit,   SIGNAL('textEdited(QString)'),   self.setDiscId)
        
        
        self.initCodepageCombo()
        self.connect(self.form.codepageCombo, SIGNAL('currentIndexChanged(int)'), self.setCodePage)

        self.connect(self.converter, SIGNAL('finished()'), self.setControlsEnable)
        self.connect(self.converter, SIGNAL('error(QString)'), self.error)
        self.connect(self.converter, SIGNAL('progress(int, QString, int)'), self.onConvertProgress)
        self.connect(self._actAbort, SIGNAL('triggered()'), self.converter.cancel)

        
        self.form.outFormatCombo.addItem(self.tr("FLAC"), QVariant(AudioFormat.FLAC))
        self.form.outFormatCombo.addItem(self.tr("OGG"),  QVariant(AudioFormat.OGG))
        self.form.outFormatCombo.addItem(self.tr("WAV"),  QVariant(AudioFormat.WAV))
        self.form.outFormatCombo.addItem(self.tr("MP3"),  QVariant(AudioFormat.MP3))

        self.loadSettings()
        self._configureDialog = None
 
 
   #****************************************************************
   #
   #****************************************************************
    def initCodepageCombo(self):
        self.form.codepageCombo.addItem(self.tr("Unicode (UTF-8)"),         QVariant("UTF-8"))
        self.form.codepageCombo.addItem(self.tr("Unicode (UTF-16LE)"),      QVariant("UTF-16LE"))
        self.form.codepageCombo.addItem(self.tr("Unicode (UTF-16BE)"),      QVariant("UTF-16BE"))
        self.form.codepageCombo.insertSeparator(99999)

        self.form.codepageCombo.addItem(self.tr("Cyrillic (Windows-1251)"), QVariant("CP1251"))
        self.form.codepageCombo.addItem(self.tr("Cyrillic (CP-866)"),       QVariant("CP866"))



   #****************************************************************
   #
   #****************************************************************
    def closeEvent(self, event):
        self.stopConvert()
        self.saveSettings()
          
          
   #****************************************************************
   #
   #****************************************************************
    def setMenu(self):
        menubar = self.menuBar()
        
        mFile = menubar.addMenu(self.tr('&File'))
        
        # Open CUE ........................................
        self._actOpenCueFile = QtGui.QAction(KIcon('document-open'), self.tr('Open CUE file'), self)
        self._actOpenCueFile.setShortcut('Ctrl+O')
        self.connect(self._actOpenCueFile, SIGNAL('triggered()'), self.openCueFile)
        mFile.addAction(self._actOpenCueFile)
        self.form.cueFileButton.setDefaultAction(self._actOpenCueFile)

        # Open audio ......................................
        self._actOpenAudioFile = QtGui.QAction(KIcon('document-open'), self.tr('Open audio file'), self)
        self._actOpenAudioFile.setShortcut('Ctrl+L')
        self.connect(self._actOpenAudioFile, SIGNAL('triggered()'), self.openAudioFile)
        mFile.addAction(self._actOpenAudioFile)
        self.form.audioFileButton.setDefaultAction(self._actOpenAudioFile)

        # Select out directory ............................
        self._actOpenOutDir = QtGui.QAction(KIcon('document-open-folder'), self.tr('Select result directory'), self)
        self.connect(self._actOpenOutDir, SIGNAL('triggered()'), self.openOutDir)
        self.form.outDirButton.setDefaultAction(self._actOpenOutDir)

        #..................................................
        mFile.addSeparator()

        # Start convert ...................................
        self._actConvert = QtGui.QAction(KIcon('arrow-right'), self.tr('Convert'), self)
        self._actConvert.setShortcut('Ctrl+W')
        self._actConvert.setToolTip(self.tr('Start conversion process.'))
        self.connect(self._actConvert, SIGNAL('triggered()'), self.startConvert)
        self._actConvert.setEnabled(False)
        mFile.addAction(self._actConvert)
        self.form.runButton.setDefaultAction(self._actConvert)


        # Stop convert ....................................
        self._actAbort = QtGui.QAction(self.tr('Abort'), self)
        self._actAbort.setShortcut('Ctrl-X')
        self._actAbort.setToolTip(self.tr('Abort conversion process.'))
        #self.connect(self._actAbort, SIGNAL('triggered()'), self.stopConvert)
        self._actAbort.setEnabled(False)
        mFile.addAction(self._actAbort)
        self.form.abortButton.setDefaultAction(self._actAbort)


        #..................................................
        mFile.addSeparator()

        # Exit ............................................
        actExit = QtGui.QAction(KIcon('application-exit'), self.tr('Exit'), self)
        actExit.setShortcut('Ctrl+Q')
        self.connect(actExit, SIGNAL('triggered()'), SLOT('close()'))
        mFile.addAction(actExit)
        

        mSettings = menubar.addMenu(self.tr('&Settings'))

        # Configure FLACON .................................
        self._actConfigure = QtGui.QAction(KIcon('configure'), self.tr('&Configure flacon'), self)
        self._actConfigure.setShortcut('Ctrl+M')
        self.connect(self._actConfigure, SIGNAL('triggered()'), self.configure)
        mSettings.addAction(self._actConfigure)

        # Configure Encoder ................................
        self._actConfigureEncoder = QtGui.QAction(KIcon('configure'), self.tr('Configure encoder'), self)
        self.connect(self._actConfigureEncoder, SIGNAL('triggered()'), self.configureEncoder)
        self.form.configureEncoderBtn.setDefaultAction(self._actConfigureEncoder)
        
        mHelp = menubar.addMenu(self.tr('&Help'))

        # About flacon ....................................
        actAbout = QtGui.QAction(KIcon('help-about'), self.tr('&About flacon'), self)
        self.connect(actAbout, SIGNAL('triggered()'), self.aboutFlacon)
        mHelp.addAction(actAbout)

        # TrackView context menu ..........................
        self._actEditTrackTitle = QtGui.QAction(self.tr('Edit &Track title'), self) 
        self.connect(self._actEditTrackTitle, SIGNAL('triggered()'), self.startEditTrackTitle)
        self._actEditTrackTitle.setShortcut('Ctrl+T')
	self.form.trackListView.addAction(self._actEditTrackTitle)        

        self._actEditArtist = QtGui.QAction(self.tr('Edit &Artist name'), self)
        self.connect(self._actEditArtist, SIGNAL('triggered()'), self.startEditArtist)
        self._actEditArtist.setShortcut('Ctrl+N')
	self.form.trackListView.addAction(self._actEditArtist)        


   #****************************************************************
   # Load settings 
   #****************************************************************
    def loadSettings(self):
        settings = QSettings("flacon", "flacon")
        
        # MainWindow geometry
        (width, ok)  = settings.value("MainWindow/Width",  QVariant(987)).toInt()
        (height, ok) = settings.value("MainWindow/Height", QVariant(450)).toInt()
        self.resize(width, height)       

        # Splitter state
        self.form.splitter.restoreState(settings.value("MainWindow/Splitter").toByteArray())
       

        # Out file pattern & dir
        s = settings.value("OutFiles/Pattern", QVariant('%a/%y - %A/%n - %t')).toString()
        self.form.outPatternEdit.setText(s)
        self.trackList.setPattern(s)
    
        s = settings.value("OutFiles/Directory", QVariant('~/music')).toString()
        self.form.outDirEdit.setText(s)
        self.trackList.setOutDir(s)

        f = settings.value("OutFiles/Format", AudioFormat.FLAC)
        n = self.form.outFormatCombo.findData(f)
        self.form.outFormatCombo.setCurrentIndex(n)
        
        self._lastDir = settings.value("Misc/LastDirectory", QVariant('~/')).toString()
 
        ## CodePage
        s = settings.value("Tags/Codepage", QVariant('UTF-8'))
        n = self.form.codepageCombo.findData(QVariant(s))
        self.setCodePage(n)
        
        qApp.preferences.load(settings)


   #****************************************************************
   # Write settings 
   #****************************************************************
    def saveSettings(self):
        settings = QSettings("flacon", "flacon")
        
        settings.setValue("MainWindow/Width", QVariant(self.size().width()))
        settings.setValue("MainWindow/Height", QVariant(self.size().height()))
        settings.setValue("MainWindow/Splitter", QVariant(self.form.splitter.saveState()))
        
        settings.setValue("OutFiles/Pattern", QVariant(self.form.outPatternEdit.text()))
        settings.setValue("OutFiles/Directory", QVariant(self.form.outDirEdit.text()))
        
        n = self.form.outFormatCombo.currentIndex()
        settings.setValue("OutFiles/Format", QVariant(self.form.outFormatCombo.itemData(n)))
        
        n = self.form.codepageCombo.currentIndex()
        settings.setValue("Tags/Codepage", QVariant(self.form.codepageCombo.itemData(n)))

        settings.setValue("Misc/LastDirectory", QVariant(self._lastDir))

        qApp.preferences.write(settings)


   #****************************************************************
   # Open treeview inline editor for track title
   #****************************************************************
    def startEditTrackTitle(self):
        index = QModelIndex(self.form.trackListView.currentIndex())
        if (index.isValid()):
            index = index.sibling(index.row(), TrackList.COLUMN_TITLE)
            self.form.trackListView.edit(index)


   #****************************************************************
   # Open treeview inline editor for artist
   #****************************************************************
    def startEditArtist(self):
        index = QModelIndex(self.form.trackListView.currentIndex())
        if (index.isValid()):
            index = index.sibling(index.row(), TrackList.COLUMN_ARTIST)
            self.form.trackListView.edit(index)


   #****************************************************************
   # 
   #****************************************************************
    def getSelectedTracks(self):
        tracks = []
        for ind in self.form.trackListView.selectionModel().selectedIndexes():
            tracks.append(self.trackList.track(ind.row()))

        if (not len(tracks)): # No tracks was selected, return all tracks
            tracks = self.trackList.tracks()

        return tracks


   #****************************************************************
   # Set artist tag for all selected tracks
   #****************************************************************
    def setArtist(self, value):
        for track in self.getSelectedTracks():
            track.setArtist(value)
            
        self.trackList.refresh()


   #****************************************************************
   # Set album tag for all selected tracks
   #****************************************************************
    def setAlbum(self, value):
        for track in self.getSelectedTracks():
            track.setAlbum(value)
            
        self.trackList.refresh()


   #****************************************************************
   # Set genre tag for all selected tracks
   #****************************************************************
    def setGenre(self, value):
        for track in self.getSelectedTracks():
            track.setGenre(value)
            
        self.trackList.refresh()


   #****************************************************************
   # Set year tag for all selected tracks
   #****************************************************************
    def setYear(self, value):
        for track in self.getSelectedTracks():
            track.setYear(value)
            
        self.trackList.refresh()


   #****************************************************************
   # Set diskId tag for all selected tracks
   #****************************************************************
    def setDiscId(self, value):
        for track in self.getSelectedTracks():
            track.setDiscId(value)
            
        self.trackList.refresh()


   #****************************************************************
   #
   #****************************************************************
    def setCodePage(self, index):
        if (index>-1):
            codec = self.form.codepageCombo.itemData(index).toString()
            self.trackList.setCodePage(codec)


   #****************************************************************
   #
   #****************************************************************
    def setOutFormat(self, index):
        format = self.form.outFormatCombo.itemData(index)

        if format == AudioFormat.FLAC: 
            self.trackList.setFileExtension('flac')
            self._actConfigureEncoder.setEnabled(True)
            
        if format == AudioFormat.OGG:  
            self.trackList.setFileExtension('ogg')
            self._actConfigureEncoder.setEnabled(True)
            
        if format == AudioFormat.MP3:  
            self.trackList.setFileExtension('mp3')
            self._actConfigureEncoder.setEnabled(True)
            
        if format == AudioFormat.WAV:  
            self.trackList.setFileExtension('wav')
            self._actConfigureEncoder.setEnabled(False)
        

   #****************************************************************
   #
   #****************************************************************
    def trackListReset(self):
        index = self.form.trackListView.currentIndex()
        if (not index.isValid()):
            index = self.trackList.index(0, 0, QModelIndex())
            self.form.trackListView.setCurrentIndex(index)
            self.form.trackListView.clearSelection()

        self.refreshEdits()

   #****************************************************************
   #
   #****************************************************************
    def refreshEdits(self):
        index = self.form.trackListView.currentIndex()
        someChanged = False

        if (index.isValid()):
            track = self.trackList.track(index.row())

            if (self.form.tagArtistEdit.text() != track.artist()):
                self.form.tagArtistEdit.setText(track.artist())
                someChanged = True

            if (self.form.tagAlbumEdit.text() !=  track.album()):
                self.form.tagAlbumEdit.setText(track.album())
                someChanged = True


            if (self.form.tagGenreEdit.text() != track.genre()):
                self.form.tagGenreEdit.setText(track.genre())
                someChanged = True

            if (self.form.tagYearEdit.value() != track.year()):
                self.form.tagYearEdit.setText(track.year())

                someChanged = True

            if (self.form.tagDiscIdEdit.text() != track.discId()):
                self.form.tagDiscIdEdit.setText(track.discId())
                someChanged = True
        else:
            self.form.tagArtistEdit.clear()
            self.form.tagAlbumEdit.clear()
            self.form.tagGenreEdit.clear()
            self.form.tagYearEdit.clear()
            self.form.tagDiscIdEdit.clear()
            
            
        if (self.form.tagStartNumEdit.value() != self.trackList.startTrackNum()):
            self.form.tagStartNumEdit.setValue(self.trackList.startTrackNum())
            someChanged = True

        if (self.form.audioFileEdit.text() != self.trackList.audioFile()):        
            self.form.audioFileEdit.setText(self.trackList.audioFile())
            someChanged = True

        if (self.form.cueFileEdit.text() != self.trackList.cueFile()):          
            self.form.cueFileEdit.setText(self.trackList.cueFile())
            someChanged = True

        if (self.form.outDirEdit.text() != self.trackList.outDir()):          
            self.form.outDirEdit.setText(self.trackList.outDir())
            someChanged = True

        n = self.form.codepageCombo.findData(QVariant(self.trackList.codePage()))
        if (self.form.codepageCombo.currentIndex() != n):          
            self.form.codepageCombo.setCurrentIndex(n)
            someChanged = True

        self.setControlsEnable()


   #****************************************************************
   #
   #****************************************************************
    def tagsChanged(self):
        self.form.trackListView.hideColumn(TrackList.COLUMN_STATUS)        
        
        
   #****************************************************************
   #
   #****************************************************************
    def openCueFile(self):
        flt = "*.cue|"             + self.tr("CUE files (*.cue)")     + "\n" + \
              "*.*|"               + self.tr("All files (*.*)")       + "\n"
        
        cueFile = KFileDialog.getOpenFileName(KUrl.fromPath(self._lastDir), flt, self, self.tr("Open CUE file"))
        if (cueFile):
            self._lastDir = QFileInfo(cueFile).dir().path()
            self.setCueFile(cueFile)


   #****************************************************************
   #
   #****************************************************************
    def openAudioFile(self):
        flt = "*.ape *.wav *flac *wv|" + self.tr("All supported formats") + "\n"  \
              "*.ape|"  + self.tr("APE files (*.ape)")     + "\n"  \
              "*.flac|" + self.tr("FLAC files (*.flac)")   + "\n"  \
              "*.wv|"   + self.tr("WavPack files (*.wv)")  + "\n"  \
              "*.wav|"  + self.tr("WAV files (*.wav)")     + "\n"  \
              "*.*|"    + self.tr("All files (*.*)")       + "\n"

        audioFile = KFileDialog.getOpenFileName(KUrl.fromPath(self._lastDir), flt, self, self.tr("Open audio file"))
        if (audioFile):
            self._lastDir = QFileInfo(audioFile).dir().path()
            self.setAudioFile(audioFile)


   #****************************************************************
   #
   #****************************************************************
    def setCueFile(self, fileName):
        try:
            QApplication.setOverrideCursor(Qt.WaitCursor)
            self.trackList.setCueFile(fileName)
            self.form.cueFileEdit.setText(fileName)
            QApplication.restoreOverrideCursor()                 
        except Exception,e:
            QApplication.restoreOverrideCursor()                 
            self.error(QString(unicode(e)))
        
        
   #****************************************************************
   #
   #****************************************************************
    def setAudioFile(self, fileName):
        try:
            QApplication.setOverrideCursor(Qt.WaitCursor)            
            self.trackList.setAudioFile(fileName)
            self.form.audioFileEdit.setText(fileName)
            QApplication.restoreOverrideCursor()                     
        except Exception,e:
            QApplication.restoreOverrideCursor()                 
            self.error(QString(unicode(e)))
        
        
   #****************************************************************
   #
   #****************************************************************
    def openOutDir(self):
        outDir = KFileDialog.getExistingDirectory(KUrl.fromPath(self.form.outDirEdit.text()), self, self.tr("Select result directory"))
        if (outDir):
            self.trackList.setOutDir(outDir)


   #****************************************************************
   #
   #****************************************************************
    def setColumnsWidth(self):
        for i in xrange(0, self.form.trackListView.header().count()):
            self.form.trackListView.resizeColumnToContents(i)
            self.form.trackListView.setColumnWidth(i, self.form.trackListView.columnWidth(i)+12)
        
        
   #****************************************************************
   #
   #****************************************************************
    def startConvert(self):
        n = self.form.outFormatCombo.currentIndex()
        format = self.form.outFormatCombo.itemData(n).toString()
        
        for track in self.trackList.tracks():    
            track.setProgress('')
        
        self.form.trackListView.showColumn(TrackList.COLUMN_STATUS)
        self.form.trackListView.setColumnWidth(1, 150)

        self.converter.start(self.trackList, format)
        self.setControlsEnable()


   #****************************************************************
   #
   #****************************************************************
    def stopConvert(self):
        self.converter.cancel()
        self.setControlsEnable()
            

   #****************************************************************
   #
   #****************************************************************
    def onConvertProgress(self, trackIndex, text, percent):
        try:
            self.trackList.track(trackIndex).setProgress(text, percent)
        except:
            pass
   

   #****************************************************************
   #
   #****************************************************************
    def setControlsEnable(self):
        running = self.converter.isRunning()
        notLoad = self.trackList.tracksCount() < 1
        
        self.form.inFilesBox.setDisabled(running)
        self.form.outFilesBox.setDisabled(running)
        self.form.tagsBox.setDisabled(running or notLoad)
        
        self._actOpenCueFile.setDisabled(running)
        self._actOpenAudioFile.setDisabled(running)
    
        self._actConfigure.setDisabled(running)
        self._actAbort.setEnabled(running)
        self._actConvert.setDisabled(running or notLoad)

        self._actEditTrackTitle.setDisabled(running or notLoad)
        self._actEditArtist.setDisabled(running or notLoad)


   #****************************************************************
   #
   #****************************************************************
    def error(self, msg):
        KMessageBox.error(None, QString(msg), self.tr("Error"))


   #****************************************************************
   #
   #****************************************************************
    def aboutFlacon(self):
        dlg = KAboutApplicationDialog(aboutData, self.form.trackListView)
        dlg.exec_()


   #****************************************************************
   #
   #****************************************************************
    def configureEncoder(self):
        format = self.form.outFormatCombo.itemData(self.form.outFormatCombo.currentIndex())
        index = -1
        if format == AudioFormat.FLAC: index = ConfigureDialog.flacPageIndex
        if format == AudioFormat.OGG:  index = ConfigureDialog.oggPageIndex
        if format == AudioFormat.MP3:  index = ConfigureDialog.mp3PageIndex
        self.configure(index) 


   #****************************************************************
   #
   #****************************************************************
    def configure(self, pageIndex=-1):
        if not self._configureDialog:
            self._configureDialog = ConfigureDialog(self)
        
        self._configureDialog.show(pageIndex)
        self._configureDialog.raise_()
        self._configureDialog.activateWindow()
        self.connect(self._configureDialog, SIGNAL('finished(int)'), self.closeConfigureDialog)


   #****************************************************************
   #
   #****************************************************************
    def closeConfigureDialog(self):
        dialog = self.sender()
        del(dialog)
        


#*******************************************************************
#
#*******************************************************************
#if __name__ == "q__main__":
#    app = QtGui.QApplication(sys.argv)

#    translator = QTranslator(app)
#    translator.load('flacon_ru')
#    app.installTranslator(translator)

#    main = flacon()
#    main.show()

#    sys.exit(app.exec_())


if __name__ == "__main__":
    binDir = os.path.dirname(os.path.realpath( __file__ ))
    sys.stdout = os.fdopen(sys.stdout.fileno(), 'w', 0)

    translator = QTranslator()
    translator.load('flacon_' + QLocale.system().name(), binDir+'/translations')
    
    appName     = "flacon"
    catalog     = ""
    programName = translator.translate('About', "flacon")
    version     = "0.2"
    description = translator.translate('About', "flacon - extracts separate tracks from audio files.")
    license     = KAboutData.License_GPL
    copyright   = translator.translate('About', "(c) 2009 Alexander Sokolov")
    text        = translator.translate('About', "Extracts audio tracks from WAV, FLAC, APE and WV files and converts them to separate FLAC, WAV, OGG, MP3  files.")
    homePage    = ""
    bugEmail    = "asokol@mail.ru"

    author1Name  = translator.translate('About Author', "Alexander Sokolov")
    author1Task  = translator.translate('About Author', "Developer")
    author1Email = "asokol@mail.ru"
    author1Web   = ""

    aboutData = KAboutData(appName, catalog, ki18n(programName.toUtf8()), version, 
                           ki18n(description.toUtf8()), license, ki18n(copyright.toUtf8()), 
                           ki18n(text.toUtf8()), homePage, bugEmail)

    aboutData.addAuthor(ki18n(author1Name.toUtf8()), ki18n(author1Task.toUtf8()), author1Email, author1Web)


    KCmdLineArgs.init(sys.argv, aboutData)

    app = KApplication()
    app.installTranslator(translator)
    qApp.preferences = Preferences()
    
    window = Flacon()
    window.show()
    sys.exit(app.exec_())
