# -*- coding: utf-8 -*-
# Project : flacon
# File : tracklist.py
# (c) Alexander Sokolov asokol@mail.ru
#
# This file is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This file is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA


import os
import re

from PyQt4 import QtGui, QtCore 
from PyQt4.QtCore import *
from PyQt4.QtGui import *

from preferences import *

class DecoderNotFound(Exception):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, decoder):
        Exception.__init__(self, decoder)


class Track():
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, trackList):
        self._trackList=trackList
        self._trackNum = -1
        
        self._artist = None
        self._cueArtist = ''
        
        self._title = None
        self._cueTitle = ''

        self._progressText = ''
        self._progress = -1

        self._album = None
        self._cueAlbum = ''

        self._genre = None
        self._cueGenre = ''

        self._year = 0
        
        self._discId = None


   #****************************************************************
   #
   #****************************************************************
    def index(self):
        return self._trackList._tracks.index(self)


   #****************************************************************
   #
   #****************************************************************
    def progress(self):
        return self._progress


   #****************************************************************
   #
   #****************************************************************
    def progressText(self):
        return QString(self._progressText)


   #****************************************************************
   #
   #****************************************************************
    def setProgress(self, text, percent=-1):
        self._progressText = QString(text)
        self._progress = percent
        self._trackList.update(self.index(), TrackList.COLUMN_STATUS)
        #self._trackList.refresh()
        #self._trackList.dc(self.index())

   #****************************************************************
   #
   #****************************************************************
    def artist(self):
        if (self._artist != None):           
            return self._artist
        else:
            return self.decode(self._cueArtist)


   #****************************************************************
   #
   #****************************************************************
    def setArtist(self, value):
        self._artist = QString(value)
        self._trackList.emitTagsChanged()     


   #****************************************************************
   #
   #****************************************************************
    def setCueArtist(self, value):
        self._cueArtist = QString(value)


   #****************************************************************
   #
   #****************************************************************
    def title(self):
        if (self._title != None):
            return self._title
        else:
            return self.decode(self._cueTitle) 


   #****************************************************************
   #
   #****************************************************************
    def setTitle(self, value):
        self._title = QString(value)
        self._trackList.emitTagsChanged()     


   #****************************************************************
   #
   #****************************************************************
    def setCueTitle(self, value):
        self._cueTitle = QString(value)


   #****************************************************************
   #
   #****************************************************************
    def album(self):
        if (self._album != None):
            return self._album
        else:
            return self.decode(self._cueAlbum) 
            
            
   #****************************************************************
   #
   #****************************************************************
    def setAlbum(self, value):
        self._album = QString(value)
        self._trackList.emitTagsChanged()     
        
   
   #****************************************************************
   #
   #****************************************************************
    def setCueAlbum(self, value):
        self._cueAlbum = QString(value)
    
    
   #****************************************************************
   #
   #****************************************************************
    def genre(self):
        if (self._genre != None): 
            return self._genre
        else: 
            return self.decode(self._cueGenre) 
    
    
   #****************************************************************
   #
   #****************************************************************
    def setGenre(self, value):
        self._genre = QString(value)
        self._trackList.emitTagsChanged()     
    
    
   #****************************************************************
   #
   #****************************************************************
    def setCueGenre(self, value):
        self._cueGenre = QString(value)
    

   #****************************************************************
   #
   #****************************************************************
    def trackNum(self):
        return int(self._trackNum)+ self._trackList.startTrackNum() - 1


   #****************************************************************
   #
   #****************************************************************
    def setCueTrackNum(self, value):
        self._trackNum = value


   #****************************************************************
   #
   #****************************************************************
    def year(self):
        return int(self._year)


   #****************************************************************
   #
   #****************************************************************
    def setYear(self, value):
        self._year = int(value)
        self._trackList.emitTagsChanged()     


   #****************************************************************
   #
   #****************************************************************
    def discId(self):
        return self._discId


   #****************************************************************
   #
   #****************************************************************
    def setDiscId(self, value):
        self._discId = QString(value)
        self._trackList.emitTagsChanged()     



   #TODO: Перевести на QString
   #****************************************************************
   # %N  Number of tracks       %n  Track number
   # %a  Artist                 %A  Album title            
   # %y  Year                   %g  Genre
   # %t  Track title
   #****************************************************************
    def fileName(self):
        pattern = self._trackList.pattern()
        if (not  pattern): 
            pattern = "%n track"

        perc = False
        res = ''
        for c in (pattern):
            if (perc):
                perc = False
                if   (c == '%'): res += '%'; perc = True 
                elif (c == 'N'): res += "%02d" % int(self._trackList.tracksCount())
                elif (c == 'n'): res += "%02d" % int(self.trackNum())
                elif (c == 'A'): res += self.safeStr(self.album())
                elif (c == 't'): res += self.safeStr(self.title())
                elif (c == 'a'): res += self.safeStr(self.artist())               
                elif (c == 'y'): 
                    if (self.year()) :
                        res += "%02d" % int(self.year())
                elif (c == 'G'): res += self.safeStr(self.genre())        
                else:            res += '%' + c
            else:
                if (c == '%'):
                    perc = True
                else:
                    res += c
        
        return QString("%1.%2").arg(res).arg(self._trackList.fileExtension())


   #****************************************************************
   #
   #****************************************************************
    def safeStr(self, str):
        res = str[:]
        res = res.replace('|', "-")
        res = res.replace('/', "-")
        res = res.replace('\\', "-")
        res = res.replace(':', "-")
        return res


   #****************************************************************
   #
   #****************************************************************
    def outFile(self):
        if (self.fileName() == ''):
            return QString('')        
        
        res = QFileInfo(self.fileName())

        if (res.isAbsolute()):
            return res

        # Add dir from trackList.outDir =================== 
        outDir = QString(self._trackList.outDir())
        if (not outDir.isEmpty()):
            if (outDir.startsWith("~/")):
                outDir.replace(0, 1, QDir.homePath())
                
            res = QFileInfo(outDir + "/" + res.filePath())

        if (res.isAbsolute()):
            return res

        # Add dir from trackList.audioFile file ===========
        af = QFileInfo(self._trackList.audioFile()).dir().absolutePath()
        
        if (not af.isEmpty()):
            res = QFileInfo(af + "/" + res.filePath())

        return res


   #****************************************************************
   #
   #****************************************************************
    def decode(self, string):
        codec = QTextCodec.codecForName(self._trackList.codePage())
        if (codec):
            return codec.toUnicode(string)
        else: 
            return string        


class TrackList(QAbstractItemModel):
    StatusPercentRole = Qt.UserRole
    StatusTextRole = StatusPercentRole + 1

    COLUMN_TRACKNUM = 0
    COLUMN_STATUS   = 1
    COLUMN_TITLE    = 2
    COLUMN_ARTIST   = 3
    COLUMN_FILENAME = 4
    COLUMNS_COUNT   = 5
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, parent=None):
        QAbstractItemModel.__init__(self, parent)
        self.clear()
        self._view=None
        self._cueFile  = QFileInfo('')
        self._cueFileAuto = True
        self._cueData = None
        self._audioFile= QFileInfo('')
        self._audioFileAuto = True
        self._pattern =  QString('')
        self._outDir =   QString('')
        self._fileExt =  QString('')
        self._codePage = 'UTF-8'
        self._audioInfo = None
        

   #****************************************************************
   #
   #****************************************************************
    def view(self):
        return self._view


   #****************************************************************
   #
   #****************************************************************
    def setView(self, value):
        self._view = value


   #****************************************************************
   #
   #****************************************************************
    def setIcon(self, icon):
        self._icon = icon


   #****************************************************************
   #
   #****************************************************************
    def clear(self):
        self._tracks = []
        self._startTrackNum = 1


   #****************************************************************
   #
   #****************************************************************
    def update(self, row, column):
        index = self.index(row, column, QModelIndex())
        self.emit(SIGNAL("dataChanged(QModelIndex, QModelIndex)"), index, index)
        
        
   #****************************************************************
   #
   #****************************************************************
    def refresh(self):
        rows = self.rowCount(QModelIndex())
        cols = self.columnCount(QModelIndex())
        index1 = self.index(0, 0, QModelIndex())
        index2 = self.index(rows-1, cols-1, QModelIndex)
        self.emit(SIGNAL("dataChanged(QModelIndex, QModelIndex)"), index1, index2)        


   #****************************************************************
   #
   #****************************************************************
    def emitTagsChanged(self):
        self.emit(SIGNAL("tagsChanged()"))     


   #****************************************************************
   #
   #****************************************************************
    def rowCount(self, parent):
        return len(self._tracks)
    

   #****************************************************************
   #
   #****************************************************************
    def columnCount(self, parent):
        return self.COLUMNS_COUNT
    

   #****************************************************************
   #
   #****************************************************************
    def headerData(self, section, orientation, role):
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            if (section == self.COLUMN_TRACKNUM): return QVariant(self.tr("Track"))
            if (section == self.COLUMN_STATUS):   return QVariant(self.tr("Status"))                
            if (section == self.COLUMN_TITLE):    return QVariant(self.tr("Title"))
            if (section == self.COLUMN_ARTIST):   return QVariant(self.tr("Artist"))
            if (section == self.COLUMN_FILENAME): return QVariant(self.tr("File"))

        return QVariant()


   #****************************************************************
   #
   #****************************************************************
    def data(self, index, role):
        if not index.isValid():
            return QVariant()
    
        track = index.internalPointer()

        # =================================================
        if (role == Qt.DisplayRole or role == Qt.EditRole ):

            if (index.column() == self.COLUMN_TRACKNUM): return QVariant("%02d" % track.trackNum())
            if (index.column() == self.COLUMN_STATUS):   return QVariant('')
            if (index.column() == self.COLUMN_TITLE):    return QVariant(track.title())
            if (index.column() == self.COLUMN_ARTIST):    return QVariant(track.artist())
            if (index.column() == self.COLUMN_FILENAME): return QVariant(track.fileName())

        # =================================================
        if (role == Qt.ToolTipRole):
            if (index.column() == self.COLUMN_FILENAME): return QVariant(track.outFile().filePath())

        # =================================================
        elif (role == Qt.DecorationRole):
            if (index.column() == self.COLUMN_TRACKNUM): return QVariant(self._icon)
                
        # =================================================
        elif (role == TrackList.StatusPercentRole):
            return track.progress()

        # =================================================
        elif (role == TrackList.StatusTextRole):
            return QVariant(track.progressText())

        return QVariant()


   #****************************************************************
   #
   #****************************************************************
    def setData(self, index, value, role):
        if not index.isValid(): 
            return 0

        if role != Qt.EditRole: 
            return 0
            
        indexes = self._view.selectionModel().selectedIndexes()
        for ind in indexes:
            track = ind.internalPointer()

            if (index.column() == self.COLUMN_TITLE):
                track.setTitle(value.toString())

            elif (index.column() == self.COLUMN_ARTIST):
                track.setArtist(value.toString())

            else:
                return False

        self.refresh()
        return True


   #****************************************************************
   #
   #****************************************************************
    def flags(self, index):
        if not index.isValid():
            return Qt.ItemIsEnabled

        res = Qt.ItemIsEnabled | Qt.ItemIsSelectable
        if (index.column() == self.COLUMN_TITLE or index.column() == self.COLUMN_ARTIST):
            res = res | Qt.ItemIsEditable
        
        return res


   #****************************************************************
   #
   #****************************************************************
    def index(self, row, column, parent):
        if row < 0 or column < 0 or row >= self.rowCount(parent) or column >= self.columnCount(parent):
            return QModelIndex()
        
        return self.createIndex(row, column, self._tracks[row])


   #****************************************************************
   #
   #****************************************************************
    def parent(self, index):
        return QModelIndex()


   #****************************************************************
   #
   #****************************************************************
    def tracks(self):
        return self._tracks[:]


   #****************************************************************
   #
   #****************************************************************
    def tracksCount(self):
        return len(self._tracks)


   #****************************************************************
   #
   #****************************************************************
    def track(self, num):
        return self._tracks[num]


   #****************************************************************
   #
   #****************************************************************
    def audioInfo(self):
        return self._audioInfo
        

   #****************************************************************
   #
   #****************************************************************
    def audioFile(self):
        return self._audioFile.filePath()


   #****************************************************************
   #
   #****************************************************************
    def setAudioFile(self, value):
        if (value == self.audioFile()): return

        self._audioFile = QFileInfo(value)
        self._audioFileAuto = False
        self._audioInfo = AudioFileInfo(self._audioFile.filePath())
        
        if (self._audioFile.isFile() and self._audioFile.isReadable()):
            if (self._cueFileAuto or self.cueFile().isEmpty()):
                self.findCueFile()

        self.reset()
        self.emit(SIGNAL("tagsChanged()"))     


   #****************************************************************
   #
   #****************************************************************
    def findAudioFile(self):
        pattern = QStringList()
        pattern << "*.ape"
        pattern << "*.wav"
        pattern << "*.flac"
        pattern << "*.wv"

        self._audioFileAuto = True
        self._audioFile = QFileInfo()
        self._audioInfo = None
        
        baseName = self._cueFile.completeBaseName()
        files = self._cueFile.dir().entryInfoList(pattern, QDir.Files | QDir.Readable)
        for f in files:
            if f.fileName().startsWith(baseName):
                try:
                    audioInfo = AudioFileInfo(f.absoluteFilePath())                
                    self._audioFile = QFileInfo(f.absoluteFilePath())
                    self._audioInfo = audioInfo
                    break
                except:
                    pass


   #****************************************************************
   #
   #****************************************************************
    def cueFile(self):
        return self._cueFile.filePath()


   #****************************************************************
   #
   #****************************************************************
    def cueData(self):
        return self._cueData


   #****************************************************************
   #
   #****************************************************************
    def setCueFile(self, value):
        if (value == self.cueFile()): return

        self._cueFile = QFileInfo(value)
        self._cueFileAuto = False
        self.clear()

        if (self._cueFile.isFile() and self._cueFile.isReadable()):
            self.loadCueFile(self.cueFile())

            if (self._audioFileAuto or self.audioFile().isEmpty()):
                self.findAudioFile()

        self.reset()
        self.emit(SIGNAL("tagsChanged()"))     


   #****************************************************************
   #
   #****************************************************************
    def findCueFile(self):
        pattern = QStringList()
        pattern << "*.cue"

        self._cueFileAuto = True
        
        baseName = self._audioFile.completeBaseName()
        files = self._audioFile.dir().entryInfoList(pattern, QDir.Files | QDir.Readable)
        for f in files:
            if f.fileName().startsWith(baseName):
                try:
                    self.setCueFile(f.absoluteFilePath())                
                    break
                except:
                    self.setCueFile('')


   #****************************************************************
   #
   #****************************************************************
    def loadCueFile(self, fileName):
        f = QFile(fileName)      
        if (f.open(QIODevice.ReadOnly | QIODevice.Text)):
            self._cueData = QStringList()    
 
            while ( not f.atEnd()):
                self._cueData.append(QString(f.readLine()))
 
            #********************************************************
            # Delete BOM 
            # see http://en.wikipedia.org/wiki/Byte_Order_Mark
            #********************************************************
            if (self._cueData[0].startsWith("\xEF\xBB\xBF")):   # UTF-8
                self._codePage = 'UTF-8'
                self._cueData[0] = self._cueData[0].remove(0, 3)
            
            if (self._cueData[0].startsWith("\xFE\xFF")):       # UTF-16 (BE)
                self._codePage = 'UTF-16BE'
                self._cueData[0] = self._cueData[0].remove(0, 2)

            if (self._cueData[0].startsWith("\xFF\xFE")):       # UTF-16 (LE)
                self._codePage = 'UTF-16LE'
                self._cueData[0] = self._cueData[0].remove(0, 2)

            # Parse CUE ...................................
            self.parseCue(self._cueData)


   #****************************************************************
   #
   #****************************************************************
    def parseCue(self, lines):
        self.clear()

        track  = QString()
        album  = QString()
        artist = QString()
        genre  = QString()
        discId = QString()
        year   = 0

        for i in xrange(0, lines.count()):
	    line = lines[i].trimmed()
	    
            if (line.isEmpty()): next
            line = line.trimmed()

            tag = line.section(' ', 0, 0).toUpper()
            value = line.section(' ', 1) 
            if (value.length()>2 and (value[0] == "'" or value[0] == '"')):
                value = value.mid(1, value.length()-2)

            #==============================================
            if (tag == "TRACK"):
                num = value.section(' ', 0, 0)
                track = Track(self)
                track.setCueTrackNum(int(num))
                track.setCueArtist(artist)
                track.setCueAlbum(album)
                track.setCueGenre(genre)
                track.setYear(year)
                track.setDiscId(discId)
                self._tracks.append(track)
                        
            #==============================================
            elif (tag == "TITLE"):
                if (track): 
                    track.setCueTitle(value)
                else: 
                    album = QString(value)
                        
            #==============================================
            elif (tag == "PERFORMER"):
                if (track): 
                    track.setCueArtist(value)
                else: 
                    artist = QString(value)
                        
            #==============================================
            elif (tag == "DATE"):           # For incorrect CUE files
                if (track): 
                    track.setYear(value)
                else: 
                    year = int(value)

            #==============================================
            elif (tag == "REM"):
                subTag = value.section(' ', 0, 0).toUpper()
                value = value.section(' ', 1) 
                if (value.length()>2 and (value[0] == "'" or value[0] == '"')):               
                    value = value.mid(1, value.length()-2)

                if (not track):
                    if (subTag == 'DATE'):   year   = int(value)
                    if (subTag == 'GENRE'):  genre  = value
                    if (subTag == 'DISCID'): discId = value


   #****************************************************************
   #
   #****************************************************************
    def pattern(self):
        return self._pattern


   #****************************************************************
   #
   #****************************************************************
    def setPattern(self, value):
        self._pattern = QString(value)
        self.refresh()
        self.emit(SIGNAL("tagsChanged()"))     


   #****************************************************************
   #
   #****************************************************************
    def outDir(self):
        return self._outDir


   #****************************************************************
   #
   #****************************************************************
    def setOutDir(self, value):
        self._outDir = QString(value)
        self.refresh()
        self.emit(SIGNAL("tagsChanged()"))     


   #****************************************************************
   #
   #****************************************************************
    def fileExtension(self):
        return self._fileExt


   #****************************************************************
   #
   #****************************************************************
    def setFileExtension(self, value):
        self._fileExt = QString(value)
        self.refresh()
        self.emit(SIGNAL("tagsChanged()"))     


   #****************************************************************
   #
   #****************************************************************
    def codePage(self):
        return self._codePage


   #****************************************************************
   #
   #****************************************************************
    def setCodePage(self, value):
        self._codePage = str(value)
        self.refresh()
        self.emit(SIGNAL("tagsChanged()"))     


   #****************************************************************
   #
   #****************************************************************
    def startTrackNum(self):
        return self._startTrackNum


   #****************************************************************
   #
   #****************************************************************
    def setStartTrackNum(self, value):
        self._startTrackNum = max(1, int(value))
        self.refresh()
        self.emit(SIGNAL("tagsChanged()"))     


   #****************************************************************
   #
   #****************************************************************
    def canConvert(self):
        return (self.tracksCount() > 0 and 
                self._audioFile.isReadable())


class AudioFileInfo(QObject):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, audioFile, parent = None):
        QObject.__init__(self, parent)
        self._items = {}
 
        if (audioFile):
            shntool = qApp.preferences.value("Programs/shntool").toString()
            proc = subprocess.Popen([unicode(shntool), 'info', unicode(audioFile)], stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
            res = proc.wait()
            if res != 0:
                msg = self.tr("Failed to open file.\nVerify that the all required programs are installed and in your preferences.")
                raise Exception(unicode(msg))
            
            for line in proc.stdout.readlines():
                name = QString(line).section(':', 0, 0).toUpper().trimmed()
                value =QString(line).section(':', 1).trimmed()
                self._items[name]=value;
            

   #****************************************************************
   #
   #****************************************************************
    def item(self, name):
        #try:
            return QString(self._items[QString(name).toUpper()])
        #except:
        #    return ""

   #****************************************************************
   #
   #****************************************************************
    def fileFormat(self):
        s = self.item('Handled by')
        if s == 'ape format module': return AudioFormat.APE
        if s == 'flac format module': return AudioFormat.APE
        if s == 'wav format module': return AudioFormat.APE
        